// SPDX-License-Identifier: GPL-2.0
/*
 * Copyright (C) 2023 Spacemit Co., Ltd.
 *
 */

#include <linux/kernel.h>
#include "../../include/spacemit_dsi_common.h"
#include "../../include/spacemit_video_tx.h"
#include <linux/delay.h>

#define UNLOCK_DELAY 0

struct spacemit_mode_modeinfo ft8201sinx101_spacemit_modelist[] = {
	{
		.name = "800x1280-60",
		.refresh = 60,
		.xres = 800,
		.yres = 1280,
		.real_xres = 800,
		.real_yres = 1280,
		.left_margin = 136,
		.right_margin = 49,
		.hsync_len = 20,
		.upper_margin = 32,
		.lower_margin = 137,
		.vsync_len = 6,
		.hsync_invert = 0,
		.vsync_invert = 0,
		.invert_pixclock = 0,
		.pixclock_freq = 88*1000,
		.pix_fmt_out = OUTFMT_RGB888,
		.width = 135,
		.height = 217,
	},
};

struct spacemit_mipi_info ft8201sinx101_mipi_info = {
	.height = 1280,
	.width = 800,
	.hfp = 49, /* unit: pixel */
	.hbp = 136,
	.hsync = 20,
	.vfp = 137, /* unit: line */
	.vbp = 32,
	.vsync = 6,
	.fps = 60,

	.work_mode = SPACEMIT_DSI_MODE_VIDEO, /*command_mode, video_mode*/
	.rgb_mode = DSI_INPUT_DATA_RGB_MODE_888,
	.lane_number = 4,
	.phy_bit_clock = 614400000,
	.phy_esc_clock = 51200000,
	.split_enable = 0,
	.eotp_enable = 0,

	.burst_mode = DSI_BURST_MODE_BURST,
};

static struct spacemit_dsi_cmd_desc ft8201sinx101_set_id_cmds[] = {
	{SPACEMIT_DSI_SET_MAX_PKT_SIZE, SPACEMIT_DSI_LP_MODE, UNLOCK_DELAY, 1, {0x01}},
};

static struct spacemit_dsi_cmd_desc ft8201sinx101_read_id_cmds[] = {
	{SPACEMIT_DSI_GENERIC_READ1, SPACEMIT_DSI_LP_MODE, UNLOCK_DELAY, 1, {0xDA}},
};

static struct spacemit_dsi_cmd_desc ft8201sinx101_set_power_cmds[] = {
	{SPACEMIT_DSI_SET_MAX_PKT_SIZE, SPACEMIT_DSI_HS_MODE, UNLOCK_DELAY, 1, {0x1}},
};

static struct spacemit_dsi_cmd_desc ft8201sinx101_read_power_cmds[] = {
	{SPACEMIT_DSI_GENERIC_READ1, SPACEMIT_DSI_HS_MODE, UNLOCK_DELAY, 1, {0xA}},
};

static struct spacemit_dsi_cmd_desc ft8201sinx101_init_cmds[] = {
	// stop reload
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0,  2, {0x41,0x5A}},
	// blank 8
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0,  3, {0x41,0x5A,0x08}},
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0,  5, {0x80,0x82,0x00,0x82,0x00}},
	// blank 9
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0,  3, {0x41,0x5A,0x09}},
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0, 17, {0x80,0x1A,0x01,0xAB,0x93,0xA5,0x97,0x59,0x2A,0xB1,0x06,0x00,0x00,0x80,0x1F,0x80,0x10}},
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0, 17, {0x90,0x42,0x44,0x84,0x14,0xC6,0x13,0x99,0x09,0xC0,0xBB,0x0A,0x0F,0x07,0xE0,0x2A,0x00}},
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0, 17, {0xA0,0x80,0x90,0x20,0xAA,0x00,0x04,0x55,0x55,0x05,0x00,0x1F,0x15,0x42,0x00,0xFE,0xFF}},
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0, 17, {0xB0,0x03,0x39,0x6D,0x01,0xD3,0x05,0xD3,0x05,0x00,0x00,0x30,0x41,0x25,0x00,0x4A,0x00}},
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0, 17, {0xC0,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x10,0x04,0x02,0x00,0x07,0x00,0x08,0x18}},
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0,  3, {0xD0,0x00,0x00}},
	// blank 10
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0,  3, {0x41,0x5A,0x0A}},
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0, 17, {0x80,0x91,0x91,0xD4,0x30,0x36,0x23,0x2D,0x34,0x3C,0x4A,0x56,0x57,0x63,0x66,0x7D,0x83}},
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0, 17, {0x90,0x6D,0x6C,0x5F,0x56,0x4A,0x3A,0x30,0x27,0x15,0x0F,0x0A,0x30,0x36,0x23,0x2D,0x34}},
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0, 17, {0xA0,0x3C,0x4A,0x56,0x57,0x63,0x66,0x7D,0x83,0x6D,0x6C,0x5F,0x56,0x4A,0x3A,0x30,0x27}},
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0,  4, {0xB0,0x15,0x0F,0x0A}},
	// blank 11
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0,  3, {0x41,0x5A,0x0B}},
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0, 17, {0x80,0x00,0x00,0x50,0xB5,0xC5,0xAC,0x00,0x00,0xA0,0x6A,0x8B,0xA9,0xAE,0x22,0x02,0x00}},
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0, 17, {0x90,0xEF,0xD8,0xB8,0xF7,0x25,0x70,0xE0,0x08,0x11,0x02,0x40,0x13,0x4F,0x4D,0x3C,0x35}},
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0,  9, {0xA0,0xF1,0xA4,0x06,0xFF,0xFF,0x00,0x03,0x00}},
	// blank 12
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0,  3, {0x41,0x5A,0x0C}},
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0, 17, {0x80,0xFA,0x68,0x90,0x01,0x28,0x05,0x7A,0x05,0x00,0x00,0x56,0x01,0x00,0x55,0x61,0x15}},
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0, 17, {0x90,0x55,0x51,0x15,0x56,0x01,0x00,0x56,0x01,0x00,0x55,0x61,0x15,0x55,0x51,0x15,0x56}},
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0, 17, {0xA0,0xB1,0x84,0x01,0x50,0x00,0x50,0x00,0x50,0x00,0x40,0x94,0x02,0x00,0x00,0x00,0x30}},
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0, 17, {0xB0,0x28,0x15,0x80,0x04,0x00,0x18,0x00,0x00,0x00,0x30,0x0A,0x00,0x00,0x00,0x00,0x00}},
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0,  8, {0xC0,0x00,0x01,0x05,0xC2,0xB0,0x00,0x00}},
	// blank 13
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0,  3, {0x41,0x5A,0x0D}},
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0, 17, {0x80,0x08,0x00,0x00,0x28,0x02,0x00,0x04,0x12,0x00,0x08,0x02,0x77,0x02,0x4A,0x10,0xF0}},
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0, 07, {0x90,0x00,0x02,0x58,0x16,0x04,0xA1}},
	// blank 14
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0,  3, {0x41,0x5A,0x0E}},
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0, 17, {0x80,0xFF,0x81,0x68,0x6C,0x22,0xFD,0x22,0x00,0x55,0x55,0x55,0x55,0x05,0x5A,0x5A,0x80}},
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0,  7, {0x90,0x01,0x00,0x00,0x60,0x01,0x00}},
	// blank 15
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0,  3, {0x41,0x5A,0x0F}},
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0, 17, {0x80,0x01,0xD5,0xC1,0x24,0x52,0x00,0x50,0x00,0x00,0x06,0x28,0x06,0x1C,0x00,0x3C,0x82}},
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0, 17, {0x90,0x1D,0x60,0x21,0x40,0x03,0x04,0x48,0xA0,0x05,0x5A,0x10,0x00,0x00,0x10,0x84,0xD0}},
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0, 17, {0xA0,0x37,0x80,0x43,0xB3,0x1D,0xC0,0x4A,0x0B,0x5A,0x71,0x22,0x80,0x19,0x4A,0x00,0x20}},
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0, 17, {0xB0,0x27,0x9E,0x01,0xC0,0x19,0x7A,0x21,0x14,0x00,0x40,0x30,0xE0,0x00,0x2D,0xD0,0x82}},
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0, 17, {0xC0,0x00,0x00,0x80,0x20,0x84,0xA0,0x21,0x1A,0x9A,0x21,0x02,0x10,0x62,0x84,0x28,0x03}},
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0, 17, {0xD0,0x8A,0x88,0x88,0x48,0x92,0x09,0x0F,0x15,0x1E,0xE0,0x31,0x06,0x04,0x02,0x14,0x04}},
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0, 17, {0xE0,0x10,0x22,0x22,0x7D,0xAD,0x05,0xD6,0x42,0xB5,0x28,0x1E,0xD2,0x62,0x40,0x41,0x00}},
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0, 16, {0xF0,0x00,0x11,0x10,0x04,0xF5,0xFF,0xFF,0x3F,0x00,0x00,0x15,0x40,0x14,0x00,0x00}},
	// blank 16
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0,  3, {0x41,0x5A,0x10}},
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0, 17, {0x80,0x00,0x00,0x03,0xE7,0x8F,0x0B,0x10,0x28,0x00,0xA9,0x66,0x0C,0x18,0xC4,0x20,0xC5}},
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0, 17, {0x90,0x1C,0x94,0xD4,0x62,0xCD,0x3D,0xF8,0x03,0x3F,0xF0,0x03,0x3F,0x00,0xFF,0xFF,0xFF}},
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0,  3, {0xA0,0x81,0x00}},
	// blank 17
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0,  3, {0x41,0x5A,0x11}},
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0,  8, {0x80,0x00,0x00,0x00,0x00,0x00,0x00,0x00}},
	// blank 18
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0,  3, {0x41,0x5A,0x12}},
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0, 17, {0x80,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00}},
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0, 17, {0x90,0x00,0x10,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00}},
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0, 17, {0xA0,0x00,0x00,0x00,0x00,0x00,0x00,0x2D,0x23,0x05,0xFB,0x00,0x2D,0x23,0x05,0xFB,0x00}},
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0,  8, {0xB0,0x01,0x00,0x00,0x00,0x00,0x00,0x00}},
	// blank 19
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0,  3, {0x41,0x5A,0x13}},
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0, 17, {0x80,0x00,0x08,0x80,0x15,0x00,0x90,0x10,0x18,0x3E,0x10,0x00,0xD6,0x03,0x04,0x18,0x00}},
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0, 17, {0x90,0x70,0x88,0x30,0x90,0x32,0x70,0x3E,0x97,0x8B,0x39,0x30,0xC4,0x09,0xC4,0x09,0xC4}},
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0, 17, {0xA0,0x89,0x4F,0xDC,0x45,0x80,0x0A,0xC4,0x09,0xC4,0x09,0x80,0x02,0x00,0x00,0x00,0x00}},
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0, 17, {0xB0,0x00,0x00,0x00,0x00,0x00,0x00,0x1B,0x01,0x31,0x75,0xB9,0x42,0x86,0xCA,0x31,0x75}},
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0, 17, {0xC0,0xB9,0x42,0x86,0xCA,0x00,0x00,0x00,0x00,0x0C,0x60,0x00,0x24,0x18,0x00,0x09,0x06}},
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0, 17, {0xD0,0x40,0x82,0x01,0x90,0x60,0x00,0x24,0xD0,0x01,0x09,0x74,0x40,0x02,0x1D,0x90,0x40}},
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0, 17, {0xE0,0x07,0x24,0xD0,0x01,0x09,0x1E,0x46,0x1E,0x46,0x00,0x00,0x20,0xC5,0x0F,0x00,0xC0}},
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0, 13, {0xF0,0x10,0x40,0x26,0x4E,0x46,0x02,0xC8,0x20,0x80,0x0C,0x00,0x00}},
	// blank 20
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0,  3, {0x41,0x5A,0x14}},
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0, 17, {0x80,0x01,0x02,0x40,0x2C,0x99,0xDF,0xEF,0xF7,0x23,0xA7,0x03,0x00,0x00,0x80,0x82,0x78}},
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0, 17, {0x90,0xF0,0xFB,0x11,0xB0,0x07,0x7E,0x3F,0x02,0x36,0xC1,0xEF,0x47,0xC0,0x26,0xF8,0xFD}},
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0, 17, {0xA0,0x08,0xD8,0x0C,0xBF,0x1F,0x01,0x9B,0xE1,0xF7,0x23,0x00,0x00,0x05,0x00,0x00,0x00}},
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0, 17, {0xB0,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x02,0x00,0x80,0x0B,0x05,0x0B}},
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0, 17, {0xC0,0x05,0x05,0x00,0x30,0x18,0x0C,0xD7,0x01,0xB2,0x97,0x04,0xC5,0x00,0x74,0x05,0x20}},
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0, 17, {0xD0,0x7B,0x49,0x50,0x0C,0x40,0x57,0x02,0xB2,0x97,0x04,0xC5,0x00,0x74,0x45,0x20,0x7B}},
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0, 17, {0xE0,0x49,0x50,0x0C,0x40,0x57,0x06,0xB2,0x97,0x04,0xC5,0x00,0x74,0x85,0x20,0x7B,0x49}},
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0, 17, {0xF0,0x50,0x0C,0x40,0x57,0x0A,0xB2,0x97,0x04,0xC5,0x00,0x74,0xC5,0x20,0x7B,0x49,0x50}},
	// blank 21
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0,  3, {0x41,0x5A,0x15}},
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0, 17, {0x80,0x0C,0x40,0x40,0x80,0xDF,0x0F,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00}},
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0, 17, {0x90,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00}},
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0, 17, {0xA0,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00}},
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0, 17, {0xB0,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00}},
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0, 17, {0xC0,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x60,0x11,0x00,0xE0,0x01,0x50,0x5A}},
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0, 17, {0xD0,0x93,0x92,0x3E,0x4E,0x41,0x0C,0x42,0x00,0xA0,0xB6,0x6D,0xDB,0xB6,0x6D,0xDB,0xB6}},
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0, 17, {0xE0,0x6D,0x5B,0xB3,0x8A,0x3E,0x2E,0x59,0x82,0xA1,0x38,0xA6,0xB6,0x6D,0xDB,0xB6,0x6D}},
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0, 17, {0xF0,0xDB,0xB6,0x6D,0x5B,0xB3,0x9A,0x3E,0x6E,0x49,0x8E,0x62,0x08,0xA2,0xB6,0x6D,0xDB}},
	// blank 22
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0,  3, {0x41,0x5A,0x16}},
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0, 17, {0x80,0xB6,0x6D,0xDB,0xB6,0x6D,0x5B,0x93,0x82,0x3E,0x0E,0x51,0x00,0x81,0x30,0xA4,0xB6}},
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0, 17, {0x90,0x6D,0xDB,0xB6,0x6D,0xDB,0xB6,0x6D,0x01,0x03,0x03,0x00,0x00,0x00,0x00,0x00,0x00}},
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0, 17, {0xA0,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xC0,0x1F,0xE1,0x01}},
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0, 17, {0xB0,0x28,0x02,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x04,0x00,0x00,0x00,0x00,0x00}},
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0, 17, {0xC0,0x00,0x00,0x00,0x00,0x00,0x00,0x02,0x02,0x00,0x00,0x00,0x00,0x00,0x00,0x02,0x00}},
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0, 17, {0xD0,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x04,0x00,0x00,0x00,0x00,0x00,0x00,0x00}},
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0, 17, {0xE0,0x00,0x00,0x00,0x00,0x01,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00}},
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0, 17, {0xF0,0x00,0x04,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x20,0x00,0x00,0x00,0x00}},
	// blank 23
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0,  3, {0x41,0x5A,0x17}},
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0, 17, {0x80,0x00,0x00,0x00,0x00,0x00,0x04,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x20}},
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0, 17, {0x90,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x10,0x00,0x00,0x00,0x00,0x00,0x00}},
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0, 17, {0xA0,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x10,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00}},
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0, 17, {0xB0,0x40,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x01,0x00,0x00,0x00,0x00,0x00}},
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0, 17, {0xC0,0x00,0x00,0x00,0x00,0x10,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00}},
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0,  7, {0xD0,0x00,0x01,0x00,0x00,0x00,0x00}},
	// blank 24
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0,  3, {0x41,0x5A,0x18}},
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0, 17, {0x80,0x7F,0x7F,0xFE,0xFE,0xFE,0xFE,0xFE,0xFE,0xFE,0xFE,0xFE,0xFE,0xD4,0x7F,0xD4,0xD4}},
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0, 17, {0x90,0xD4,0xD4,0xD4,0xD4,0xD4,0xD4,0xD4,0xD4,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA}},
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0, 17, {0xA0,0xAA,0xAA,0xAA,0xAA,0x7F,0x7F,0x7F,0x7F,0x7F,0x7F,0x7F,0x7F,0x7F,0x7F,0x7F,0x7F}},
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0, 17, {0xB0,0x7C,0x83,0x79,0x86,0x79,0x86,0x79,0x86,0x79,0x86,0x79,0x86,0x7A,0x83,0x7A,0x85}},
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0, 17, {0xC0,0x7A,0x85,0x7A,0x85,0x7A,0x85,0x7A,0x85,0x7B,0x84,0x7B,0x84,0x7B,0x84,0x7B,0x84}},
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0, 17, {0xD0,0x7B,0x84,0x7B,0x84,0x7C,0x83,0x7C,0x83,0x7C,0x83,0x7C,0x83,0x7C,0x83,0x7C,0x83}},
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0, 17, {0xE0,0xFE,0x7F,0x7F,0xFE,0xFE,0xFE,0xFE,0xFE,0xFE,0xFE,0xFE,0xFE,0xFE,0xD4,0x7F,0xD4}},
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0, 17, {0xF0,0xD4,0xD4,0xD4,0xD4,0xD4,0xD4,0xD4,0xD4,0xD4,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA,0xAA}},
	// blanbk 25
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0,  3, {0x41,0x5A,0x19}},
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0, 17, {0x80,0xAA,0xAA,0xAA,0xAA,0xAA,0x7F,0x7F,0x7F,0x7F,0x7F,0x7F,0x7F,0x7F,0x7F,0x7F,0x7F}},
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0, 17, {0x90,0x7F,0x7C,0x83,0x79,0x86,0x79,0x86,0x79,0x86,0x79,0x86,0x79,0x86,0x7A,0x83,0x7A}},
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0, 17, {0xA0,0x85,0x7A,0x85,0x7A,0x85,0x7A,0x85,0x7A,0x85,0x7B,0x84,0x7B,0x84,0x7B,0x84,0x7B}},
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0, 17, {0xB0,0x84,0x7B,0x84,0x7B,0x84,0x7C,0x83,0x7C,0x83,0x7C,0x83,0x7C,0x83,0x7C,0x83,0x7C}},
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0, 17, {0xC0,0x83,0x7F,0x04,0xFF,0x1E,0x00,0x08,0x10,0x18,0x20,0x28,0x30,0x38,0xFC,0x00,0x04}},
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0, 12, {0xD0,0x20,0x00,0x01,0x08,0x40,0x00,0x02,0x10,0x80,0x00,0xFC}},
	// blank 26
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0,  3, {0x41,0x5A,0x1A}},
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0, 17, {0x80,0xF0,0xD9,0xC8,0xBA,0xAF,0xA6,0x9E,0x98,0x92,0x8D,0x88,0x84,0x00,0x60,0xF6,0xCF}},
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0, 17, {0x90,0xFC,0x6F,0xF6,0xEF,0xCF,0xAF,0x5F,0x55,0x00,0xF8,0x00,0xF8,0x00,0xF8,0xAD,0x8D}},
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0, 17, {0xA0,0x73,0x60,0x53,0x4A,0x43,0x7F,0x78,0x10,0xA0,0x3F,0x00,0x00,0x00,0x00,0x00,0x00}},
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0, 17, {0xB0,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00}},
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0,  9, {0xC0,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00}},
	// blank 28
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0,  3, {0x41,0x5A,0x1C}},
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0, 17, {0x80,0x00,0x04,0x08,0x0C,0x00,0x10,0x14,0x18,0x1C,0x00,0x20,0x28,0x30,0x38,0x00,0x40}},
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0, 17, {0x90,0x48,0x50,0x58,0x00,0x60,0x68,0x70,0x78,0x00,0x80,0x88,0x90,0x98,0x00,0xA0,0xA8}},
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0, 17, {0xA0,0xB0,0xB8,0x00,0xC0,0xC8,0xD0,0xD8,0x00,0xE0,0xE8,0xF0,0xF8,0x00,0xFC,0xFE,0xFF}},
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0, 17, {0xB0,0x00,0x00,0x04,0x08,0x0C,0x00,0x10,0x14,0x18,0x1C,0x00,0x20,0x28,0x30,0x38,0x00}},
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0, 17, {0xC0,0x40,0x48,0x50,0x58,0x00,0x60,0x68,0x70,0x78,0x00,0x80,0x88,0x90,0x98,0x00,0xA0}},
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0, 17, {0xD0,0xA8,0xB0,0xB8,0x00,0xC0,0xC8,0xD0,0xD8,0x00,0xE0,0xE8,0xF0,0xF8,0x00,0xFC,0xFE}},
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0, 17, {0xE0,0xFF,0x00,0x00,0x04,0x08,0x0C,0x00,0x10,0x14,0x18,0x1C,0x00,0x20,0x28,0x30,0x38}},
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0, 17, {0xF0,0x00,0x40,0x48,0x50,0x58,0x00,0x60,0x68,0x70,0x78,0x00,0x80,0x88,0x90,0x98,0x00}},
	// blank 29
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0,  3, {0x41,0x5A,0x1D}},
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0, 17, {0x80,0xA0,0xA8,0xB0,0xB8,0x00,0xC0,0xC8,0xD0,0xD8,0x00,0xE0,0xE8,0xF0,0xF8,0x00,0xFC}},
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0,  5, {0x90,0xFE,0xFF,0x00,0x00}},
	// blank 31
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0,  3, {0x41,0x5A,0x1F}},
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0, 17, {0x80,0x5A,0x0F,0x70,0x09,0x28,0x30,0x38,0x40,0x48,0x50,0x58,0x60,0x20,0x30,0x38,0x40}},
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0, 17, {0x90,0x48,0x50,0x58,0x60,0x20,0x28,0x38,0x40,0x48,0x50,0x58,0x60,0x20,0x28,0x30,0x40}},
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0, 17, {0xA0,0x48,0x50,0x58,0x60,0x20,0x28,0x30,0x38,0x48,0x50,0x58,0x60,0x20,0x28,0x30,0x38}},
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0, 17, {0xB0,0x40,0x50,0x58,0x60,0x20,0x28,0x30,0x38,0x40,0x48,0x58,0x60,0x20,0x28,0x30,0x38}},
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0, 17, {0xC0,0x40,0x48,0x50,0x60,0x20,0x28,0x30,0x38,0x40,0x48,0x50,0x58,0xFF,0x5A,0x1E,0x20}},
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0, 13, {0xD0,0x00,0x02,0x20,0x00,0x02,0x20,0x00,0x02,0x20,0x00,0x02,0xFE}},
	// blank 32
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0,  3, {0x41,0x5A,0x20}},
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0, 17, {0x80,0x44,0x73,0x03,0x00,0xCA,0xF3,0xFF,0x6B,0x10,0x08,0x02,0x08,0x03,0x41,0x18,0x30}},
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0, 17, {0x90,0x18,0x44,0xE1,0x40,0xE1,0x20,0x30,0x18,0xC8,0x21,0x41,0x01,0x39,0x13,0x44,0x42}},
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0, 17, {0xA0,0x91,0x50,0x28,0x14,0x0A,0x85,0x03,0xD2,0xB0,0x80,0x36,0x7F,0xD0,0x1F,0x12,0xE0}},
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0, 17, {0xB0,0x18,0x36,0x1B,0x11,0x04,0x20,0x02,0xD0,0x80,0xB3,0x21,0x38,0x20,0x18,0x88,0x03}},
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0, 17, {0xC0,0x82,0x01,0x41,0x70,0x40,0x30,0x8D,0x0D,0xA4,0xA2,0x00,0x80,0x04,0x38,0x00,0x50}},
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0, 17, {0xD0,0x80,0x13,0xE0,0x00,0x14,0xC0,0x70,0x40,0x00,0xE0,0x81,0x00,0x00,0x28,0x0E,0x06}},
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0, 17, {0xE0,0x43,0x55,0x55,0x55,0x55,0x55,0xD5,0xCC,0xCC,0xCC,0xCC,0xCC,0x4C,0x08,0x82,0xC6}},
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0, 17, {0xF0,0xE0,0x01,0x00,0x00,0x00,0x21,0x42,0x88,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x20}},
	// blank 33
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0,  3, {0x41,0x5A,0x21}},
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0, 17, {0x80,0x04,0x41,0x06,0x49,0x90,0x04,0x49,0x00,0x00,0x49,0x00,0x00,0x49,0x00,0x00,0x49}},
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0, 17, {0x90,0x90,0x04,0x00,0x85,0x11,0x0C,0x00,0x00,0x40,0x00,0x00,0x00,0x00,0x16,0x00,0xA5}},
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0, 17, {0xA0,0x00,0x08,0x08,0x15,0x10,0xD1,0xB6,0x45,0x92,0x8A,0x20,0xA2,0x6D,0x0B,0x00,0x54}},
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0, 17, {0xB0,0x40,0x44,0xDB,0x16,0x00,0x48,0x92,0x24,0x09,0xE0,0x0F,0x80,0x55,0x30,0x01,0x10}},
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0, 17, {0xC0,0x01,0x42,0x82,0x11,0x20,0x02,0x10,0x10,0x01,0x08,0x88,0x00,0x04,0x44,0x00,0x02}},
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0, 17, {0xD0,0x22,0x00,0x04,0x91,0x80,0x1F,0x00,0x40,0x40,0x04,0x00,0x00,0x00,0x18,0x10,0x16}},
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0,  3, {0xE0,0x10,0x00}},
	// blank 34
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0,  3, {0x41,0x5A,0x22}},
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0, 17, {0x80,0x29,0xD3,0x00,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x55,0x1F,0x70}},
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0,  4, {0x90,0xF0,0x00,0x00}},
	// blank 35
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0,  3, {0x41,0x5A,0x23}},
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0, 17, {0x80,0x09,0x01,0x02,0x05,0x00,0x05,0x04,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00}},
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0, 10, {0x90,0x00,0xFF,0x1F,0x20,0x00,0x01,0x00,0x00,0xFE}},
	// blank 36
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0, 3, {0x41,0x5A,0x24}},
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0, 17, {0x80,0x00,0x03,0x00,0xFF,0xFF,0x66,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x5B,0x26}},
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0, 15 , {0x90,0x5A,0xA5,0x5A,0x00,0x00,0x00,0x00,0x22,0x22,0x3A,0x22,0x00,0x00,0x00}},
	// spi finish
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0,  3, {0x41,0x5A,0x24}},
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0,  2, {0x90,0xA5}},
	// blank select 2f
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0,  3, {0x41,0x5A,0x2F}},
	// cancel tp i2c mode
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0,  2, {0x4C,0x03}},
	// bypass or fifi mode
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0,  2, {0x09,0x01}},
	// ledpwm
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0,  3, {0x51,0xff,0x0f}},
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0,  2, {0x53,0x24}},
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0,  2, {0x55,0x00}},
	// bist mode
	// {SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 0,   2, {0x02,0x5A}},
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 120, 2, {0x11, 0x00}},
	{SPACEMIT_DSI_DCS_LWRITE, SPACEMIT_DSI_LP_MODE, 50,  2, {0x29, 0x00}},
};

static struct spacemit_dsi_cmd_desc ft8201sinx101_sleep_out_cmds[] = {
	{SPACEMIT_DSI_DCS_SWRITE,SPACEMIT_DSI_LP_MODE,120,2,{0x11, 0x00}},
	{SPACEMIT_DSI_DCS_SWRITE,SPACEMIT_DSI_LP_MODE,50,2,{0x29, 0x00}},
};

static struct spacemit_dsi_cmd_desc ft8201sinx101_sleep_in_cmds[] = {
	{SPACEMIT_DSI_DCS_SWRITE,SPACEMIT_DSI_LP_MODE,50,2,{0x28, 0x00}},
	{SPACEMIT_DSI_DCS_SWRITE,SPACEMIT_DSI_LP_MODE,120,2,{0x10, 0x00}},
};


struct lcd_mipi_panel_info lcd_ft8201sinx101 = {
	.lcd_name = "ft8201sinx101",
	.lcd_id = 0x0611,
	.panel_id0 = 0x06,
	.power_value = 0x1c,
	.panel_type = LCD_MIPI,
	.width_mm = 135,
	.height_mm = 217,
	.dft_pwm_bl = 128,
	.set_id_cmds_num = ARRAY_SIZE(ft8201sinx101_set_id_cmds),
	.read_id_cmds_num = ARRAY_SIZE(ft8201sinx101_read_id_cmds),
	.init_cmds_num = ARRAY_SIZE(ft8201sinx101_init_cmds),
	.set_power_cmds_num = ARRAY_SIZE(ft8201sinx101_set_power_cmds),
	.read_power_cmds_num = ARRAY_SIZE(ft8201sinx101_read_power_cmds),
	.sleep_out_cmds_num = ARRAY_SIZE(ft8201sinx101_sleep_out_cmds),
	.sleep_in_cmds_num = ARRAY_SIZE(ft8201sinx101_sleep_in_cmds),
	//.drm_modeinfo = ft8201sinx101_modelist,
	.spacemit_modeinfo = ft8201sinx101_spacemit_modelist,
	.mipi_info = &ft8201sinx101_mipi_info,
	.set_id_cmds = ft8201sinx101_set_id_cmds,
	.read_id_cmds = ft8201sinx101_read_id_cmds,
	.set_power_cmds = ft8201sinx101_set_power_cmds,
	.read_power_cmds = ft8201sinx101_read_power_cmds,
	.init_cmds = ft8201sinx101_init_cmds,
	.sleep_out_cmds = ft8201sinx101_sleep_out_cmds,
	.sleep_in_cmds = ft8201sinx101_sleep_in_cmds,
	.bitclk_sel = 3,
	.bitclk_div = 1,
	.pxclk_sel = 2,
	.pxclk_div = 6,
};

int lcd_ft8201sinx101_init(void)
{
	int ret;

	ret = lcd_mipi_register_panel(&lcd_ft8201sinx101);
	return ret;
}
